/*
  configuration.h - Config file

  Part of DiyouPCBFirm
  Based on Marlin & Grbl

Copyright (c) 2013 V.Ayllon.  All rights reserved.

DiyouPCBFirm is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the GNU General Public License, Version 2 or later, for full details.

Everyone is granted permission to copy, modify and redistribute
DiyouPCBFirm, but only under the conditions described in the GNU General Public
License.  A copy of this license is supposed to have been
given to you along with DiyouPCB so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/


#ifndef CONFIGURATION_H
#define CONFIGURATION_H

// This configurtion file contains the basic settings.
// Advanced settings can be found in Configuration_adv.h 
// BASIC SETTINGS: select your board type, temperature sensor type, axis scaling, and endstop configuration

//User specified version info of this build to display in [Pronterface, etc] terminal window during startup.
//Implementation of an idea by Prof Braino to inform user that any changes made
//to this build by the user have been successfully uploaded into firmware.
#define STRING_VERSION_CONFIG_H __DATE__ " " __TIME__ // build date and time
#define STRING_CONFIG_H_AUTHOR "(V.Ayllon, V1.0)" //Who made the changes.

// SERIAL_PORT selects which serial port should be used for communication with the host.
// This allows the connection of wireless adapters (for instance) to non-default port pins.
// Serial port 0 is still used by the Arduino bootloader regardless of this setting.
#define SERIAL_PORT 0

// This determines the communication speed of the printer
//#define BAUDRATE 250000 
#define BAUDRATE 115200 

//===========================================================================
//=============================Mechanical Settings===========================
//===========================================================================

// Uncomment the following line to enable CoreXY kinematics
// #define COREXY

// corse Endstop Settings
#define ENDSTOPPULLUPS // Comment this out (using // at the start of the line) to disable the endstop pullup resistors

#ifndef ENDSTOPPULLUPS
  // fine Enstop settings: Individual Pullups. will be ignord if ENDSTOPPULLUPS is defined
  #define ENDSTOPPULLUP_XMIN
  #define ENDSTOPPULLUP_YMIN
#endif

#ifdef ENDSTOPPULLUPS
  #define ENDSTOPPULLUP_XMIN
  #define ENDSTOPPULLUP_YMIN
#endif

// The pullups are needed if you directly connect a mechanical endswitch between the signal and ground pins.
const bool X_ENDSTOPS_INVERTING =true; // set to true to invert the logic of the endstops. 
const bool Y_ENDSTOPS_INVERTING =true; // set to true to invert the logic of the endstops. 

// For Inverting Stepper Enable Pin (Active Low) use 0, Non Inverting (Active High) use 1
#define MOTORS_ENABLE_ON 0

// Disables axis when it's not being used.
#define DISABLE_MOTORS false

#define INVERT_X_DIR true    // Set to true
#define INVERT_Y_DIR true    // Set to true

// ENDSTOP SETTINGS:
// Sets direction of endstops when homing; 1=MAX, -1=MIN

#define X_HOME_DIR -1
#define Y_HOME_DIR -1

#define min_software_endstops true //If true, axis won't move to coordinates less than HOME_POS.
#define max_software_endstops true  //If true, axis won't move to coordinates greater than the defined lengths below.

// How many mm add to home position to get the window's glass

#define HOMING_OFFSET_X  5+3
#define HOMING_OFFSET_Y  0+2

// How many mm add to home position to get focus position

#define FOCUS_OFFSET_X  0
#define FOCUS_OFFSET_Y  3

// Travel limits after homing

#define X_MAX_POS 212
#define X_MIN_POS 0
#define Y_MAX_POS 170
#define Y_MIN_POS 0

#define X_MAX_LENGTH (X_MAX_POS - X_MIN_POS)
#define Y_MAX_LENGTH (Y_MAX_POS - Y_MIN_POS)

// The position of the homing switches
//#define MANUAL_HOME_POSITIONS  // If defined, MANUAL_*_HOME_POS below will be used

//// MOVEMENT SETTINGS
#define NUM_AXIS 2 // The axis order in all axis related arrays is X, Y
#define HOMING_FEEDRATE		50 // set the homing speeds (mm/sec)
// default settings 

// X, Y hysteresis in mm. 
// These are the extra distances that are performed when an axis changes direction to compensate for any mechanical hysteresis.
#define X_DEFAULT_BACKLASH_MM   0.32f
#define Y_DEFAULT_BACKLASH_MM   0.18f // X, Y backlash comp. in mm
// Feed rates

#define DEFAULT_FEEDRATE	10 // Default feed rate (mm/sec)
#define DEFAULT_TRAVELRATE	50 // Default travel rate (mm/sec)

#define MAX_FEEDRATE		250 // Max feed rate (mm/sec)
#define DEFAULT_AXIS_STEPS_PER_UNIT   {80.00, 80.00} //Belt 2xNEMA16
#define DEFAULT_MAX_FEEDRATE          {MAX_FEEDRATE, MAX_FEEDRATE}    // (mm/sec)    

#define DEFAULT_MAX_ACCELERATION      3000    // 9000 Maximum start speed for accelerated moves.
#define DEFAULT_ACCELERATION          3000     // 3000 Max acceleration in mm/s^2 for printing moves 
// 
#define DEFAULT_XYJERK                10.0    // (mm/sec)

#define MAX_LASER_POWER                255    // Pwm control with this max value

#define DEFAULT_ENTRY_POWER            65     // Entry laser power
#define DEFAULT_LASER_POWER            88     // Default laser power 88
#define DEFAULT_EXIT_POWER             65     // Exit laser power

#define SMALL_CIRCLE_FEEDRATE		   10     // mm/seg
#define SMALL_CIRCLE_ENTRY_POWER	   60
#define SMALL_CIRCLE_POWER			   66     // Default laser power for vias, pads, etc.
#define SMALL_CIRCLE_EXIT_POWER	       60

#define SMALL_SEGMENT_SIZE             1      // Segments less than this in mm are small segments
#define SMALL_SEGMENT_ENTRY_POWER      69     // Entry laser power for small segments. 
#define SMALL_SEGMENT_POWER			   73     // Nominal laser power for small segments.
#define SMALL_SEGMENT_EXIT_POWER       69     // Exit laser power for small segments.

#define CMD_STATUS_REPORT '?'				  // Prints the status of the printer
#define CMD_FEED_HOLD '!'					  // Pause the printing
#define CMD_CYCLE_START '|'					  // Restart
#define CMD_RESET 0x18						  // ctrl-x reset the printer

#include "Configuration_adv.h"

#endif //__CONFIGURATION_H
