/*
Copyright (c) 2013 V.Ayllon.  All rights reserved.
 Part of DiyouPCBFirm

 Based on HP2xx:
 
 Parts Copyright (c) 1991 - 1994 Heinz W. Werntges.  All rights reserved.
 Parts Copyright (c) 1998-2002,2003 Martin Kroeker  All rights reserved.
 
 DiyouPCBFirm is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 to anyone for the consequences of using it or for whether it serves any
 particular purpose or works at all, unless he says so in writing.  Refer
 to the GNU General Public License, Version 2 or later, for full details.
 
 Everyone is granted permission to copy, modify and redistribute
 DiyouPCBFirm, but only under the conditions described in the GNU General Public
 License.  A copy of this license is supposed to have been
 given to you along with DiyouPCB so you can know your rights and
 responsibilities.  It should be in a file named COPYING.  Among other
 things, the copyright notice and this notice must be preserved on all
 copies.
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
#ifndef __AVR_ATmega328P__
	#define __AVR_ATmega328P__
#endif

#include "DiyouPCBFirm.h"
#include "Configuration.h"
#include "hpgl_parser.h"

float numpar[MAX_PARAMETERS];
char* buffptr;
uint8_t buffinx = 0;


float P1[NUM_AXIS] = { 
  P1X_default, P1Y_default};	// Scaling points
float P2[NUM_AXIS] = { 
  P2X_default, P2Y_default};
float S1[NUM_AXIS] = { 
  P1X_default, P1Y_default};	// Scaled points
float S2[NUM_AXIS] = { 
  P2X_default, P2Y_default};
float Q[NUM_AXIS] = {
  1.,1.}; 		/* Delta-P/Delta-S: Initialized with first SC   */

/** hpgl_init() - Init variables
 ** HP-GL commands, reads parameters (if expected), and initiates processing of these
 ** commands.
 **/
void hpgl_init(){

  P1[X_COORD] =  P1X_default;
  P1[Y_COORD] =  P1Y_default;
  P2[X_COORD] =  P2X_default; 
  P2[Y_COORD] =  P2Y_default;

  S1[X_COORD] =  P1X_default;
  S1[Y_COORD] =  P1Y_default;
  S2[X_COORD] =  P2X_default; 
  S2[Y_COORD] =  P2Y_default;


  Q[X_COORD] = 1.; 		/* Delta-P/Delta-S: Initialized with first SC   */
  Q[Y_COORD] = 1.;

}
/** hpgl_parser_run() - Reads the input stream character-by-character, identifies
 ** HP-GL commands, reads parameters (if expected), and initiates processing of these
 ** commands.
 **/
uint16_t hpgl_parser_run(char* buffer){
  buffptr = buffer;
  buffinx = 0;
  char c;
  uint16_t cmd;

  while((c = buffptr[buffinx++]) != '\0') {
    if ((c < '@') || (c > 'z')
      || ((c > 'Z') && (c < 'a')))
      continue;
    cmd = c << 8;
    if ((c = buffptr[buffinx++]) == '\0')
      break;	// Partial command
    if ((c < '@') || (c > 'z')
      || ((c > 'Z') && (c < 'a'))) {
      continue;
    }
    cmd |= (c & 0xFF);
    cmd &= 0xDFDF; // Pass to upper
    //			sprintf(pbuffer, "Command: #%x",cmd);
    //			print(pbuffer);
    return hpgl_parse_command(cmd);
  }
  return PARSER_ERROR;
}

/**
** hpgl_parse_command() - each command consists of 2 characters
** This function returns the command detected.
**/
uint16_t hpgl_parse_command(uint16_t command){

	switch (command) {
	case HP_AA: 		/* Arc Absolute                 */
		if (get_AA_coordinates())
			return PARSER_ERROR;
		break;
	case HP_PA: 		/* Plot Absolute                */
		if (get_PA_coordinates())
			return PARSER_ERROR;
		break;
	case HP_PD: 		/* Pen  Down                    */
	case HP_SP: 		/* Select pen */
	case HP_PU: 		/* Pen  Up                      */
	case STCMD:		   /* Status retporting */
		break;
	case HP_IP: 		/* Input reference Points P1,P2 */
		if (get_IP_coordinates())
			return PARSER_ERROR;
		break;
	case HP_SC: 		/* Input Scale Points S1,S2     */
		if (get_SC_coordinates())
			return PARSER_ERROR;
		break;
	case HP_IN: 		/* Initialize */
		break;
	case LCMD:		   /* @L Set laser on/off (not HPGL) */
	case PCMD:		   /* @P Set laser power (not HPGL)  */
	case HP_VS:        /* Velocity select */
	case HP_LO:        /* Laser ON*/
		if (get_single_value())
			return PARSER_ERROR;
		break;
	case HYCMD:		   /* @H Set Axis histeresis (BL) */
		if (get_HY_values())
			return PARSER_ERROR;
		break;
	default: 		/* Skip unknown HPGL command: */
		break;
	}

	return command;
}
/**
 ** get_IP_coordinates() - Gets IP parameters
 **
 **/
bool get_IP_coordinates(){

	if(!read_float(buffptr, &buffinx, &numpar[IP_P1X])) { 
		P1[X_COORD] = P1X_default;
		P1[Y_COORD] = P1Y_default;
		P2[X_COORD] = P2X_default;
		P2[Y_COORD] = P2Y_default;

		// Set S1
		goto Exit;
	}
	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[IP_P1Y]))    
		return true; /* x without y! */
	if(buffptr[buffinx++] != ',') { return(true); }
	if(!read_float(buffptr, &buffinx, &numpar[IP_P2X])) {  	/* No number found  */
		P2[X_COORD] += numpar[IP_P1X] - P1[X_COORD];
		P2[Y_COORD] += numpar[IP_P1Y] - P1[Y_COORD];

		P1[X_COORD] = numpar[IP_P1X];
		P1[Y_COORD] = numpar[IP_P1Y];
		goto Exit;
	}
	if(buffptr[buffinx++] != ',') { return(true); }
	if(!read_float(buffptr, &buffinx, &numpar[IP_P2Y]))    
		return true; /* x without y! */

	P1[X_COORD] = numpar[IP_P1X];
	P1[Y_COORD] = numpar[IP_P1Y];
	P2[X_COORD] = numpar[IP_P2X];
	P2[Y_COORD] = numpar[IP_P2Y];

Exit:
	// Set S1
	S1[X_COORD] = P1[X_COORD];
	S1[Y_COORD] = P1[Y_COORD];
	S2[X_COORD] = P2[X_COORD];
	S2[Y_COORD] = P2[Y_COORD];

	Q[X_COORD] = (P2[X_COORD] - P1[X_COORD]) / (S2[X_COORD] - S1[X_COORD]);
	Q[Y_COORD] = (P2[Y_COORD] - P1[Y_COORD]) / (S2[Y_COORD] - S1[Y_COORD]);

	return false;
}

/**
 ** get_SC_coordinates() - Gets SC parameters
 **
 **/
bool get_SC_coordinates(){

	if(!read_float(buffptr, &buffinx, &numpar[SC_P1X])) {     
		S1[X_COORD] = P1X_default;
		S1[Y_COORD] = P1Y_default;
		S2[X_COORD] = P2X_default;
		S2[Y_COORD] = P2Y_default;
		Q[X_COORD] = Q[Y_COORD] = 1.0;
		return false;
	}
	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[SC_P2X]))	 

		return true; /* x without y! */
	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[SC_P1Y]))      	/* No number found  */
		return true; /* x without y! */

	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[SC_P2Y]))  	     
		return true; /* x without y! */


	if (numpar[SC_P1X] == numpar[SC_P2X] || numpar[SC_P1Y] == numpar[SC_P2Y]) { 	/* min must differ from max */
		Q[X_COORD] = Q[Y_COORD] = 1.0;
		return false;
	}

	// Set S1

	S1[X_COORD] = numpar[SC_P1X];
	S1[Y_COORD] = numpar[SC_P1Y];

	S2[X_COORD] = numpar[SC_P2X];
	S2[Y_COORD] = numpar[SC_P2Y];

	// Update Delta

	Q[X_COORD] = (P2[X_COORD] - P1[X_COORD]) / (S2[X_COORD] - S1[X_COORD]);
	Q[Y_COORD] = (P2[Y_COORD] - P1[Y_COORD]) / (S2[Y_COORD] - S1[Y_COORD]);

	return false;
}


/**
 ** get_single_value() - Gets a single value
 **/
bool get_single_value(){
	if(!read_float(buffptr, &buffinx, &numpar[0]))  	     
		return true;
	return false;
}
/**
 ** get_PA_coordinates() - Gets the coordinates of a move command
 **/
bool get_PA_coordinates()
{
	if(!read_float(buffptr, &buffinx, &numpar[X_COORD])) 
		return true;

	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[Y_COORD]))
		return true;

	numpar[X_COORD] = hpgl_plotter_to_user(numpar[X_COORD],X_COORD);
	numpar[Y_COORD] = hpgl_plotter_to_user(numpar[Y_COORD],Y_COORD);
	return false;
}
/**
 * get_AA_coordinates() - Gets the arc command coordinates
 **/
bool get_AA_coordinates()
{
	if(!read_float(buffptr, &buffinx, &numpar[I_COORD]))  	      
		return true;
	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[J_COORD]))  	      
		return true;

	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[ARC_THETA]))  	      
		return true;

	buffinx++; // Opcional parameter

	read_float(buffptr, &buffinx, &numpar[SEG_ALPHA]);// Opcional

	numpar[I_COORD] = hpgl_plotter_to_user(numpar[I_COORD],I_COORD);
	numpar[J_COORD] = hpgl_plotter_to_user(numpar[J_COORD],J_COORD);

	return false;
}
/**
 ** get_HY_values() - Gets the hysteresis values
 **/
bool get_HY_values()
{
	if(!read_float(buffptr, &buffinx, &numpar[X_COORD])) 
		return true;

	if(buffptr[buffinx++] != ',') { return(true); }

	if(!read_float(buffptr, &buffinx, &numpar[Y_COORD]))
		return true;

	return false;
}
/**
 ** hpgl_user_to_plotter() - Translate user to plotter coordinates
 ** Is so basic in order to maintain compatibility with Processing.
 **/
float hpgl_user_to_plotter(float coord, uint8_t axis)
{
  if (axis > NUM_AXIS)
    return -1;
  return (P1[axis] + (coord - S1[axis]) * Q[axis])/PLOT_UNIT;
}
/**
 ** hpgl_plotter_to_user() - Translate plotter to user coordinates
 ** Is so basic in order to maintain compatibility with Processing.
 **/
float hpgl_plotter_to_user(float coord, uint8_t axis)
{
  if (axis > NUM_AXIS)
    return -1;
  return  (S1[axis] + (coord - P1[axis])/Q[axis])*PLOT_UNIT;
}

// Extracts a floating point value from a string. The following code is based loosely on
// the avr-libc strtod() function by Michael Stumpf and Dmitry Xmelkov and many freely
// available conversion method examples, but has been highly optimized for Grbl. For known
// CNC applications, the typical decimal value is expected to be in the range of E0 to E-4.
// Scientific notation is officially not supported by g-code, and the 'E' character may
// be a g-code word on some CNC systems. So, 'E' notation will not be recognized. 
// NOTE: Thanks to Radu-Eosif Mihailescu for identifying the issues with using strtod().
/**
int read_float(char *line, uint8_t *char_counter, float *float_ptr)                  
{
  #define MAX_BUF_SIZE 10

  char *ptr = line + *char_counter;
  unsigned char c;
    
  char buf[MAX_BUF_SIZE+1];
  uint16_t i = 0;
 
  while((c = *ptr++) != '\0'){
    if (c == ' ' || c == '\t' ||c == '\n' || c== '\r')
      continue;
    if (((c >= '0') && (c <= '9')) || (c == '.') || (c == '+') || (c == '-')){
		buf[i] = c;
		if (i < MAX_BUF_SIZE){
			i++;
		}
		else {
			i=0;
			break;
		}
    } 
    else { // end of number
      buf[i] = '\0';
	  break;
    }
  }

  *char_counter = ptr - line - 1; // Set char_counter to next statement

  if (i > 0){
	  *float_ptr = (float)atof(buf);
	  return true;
  }
  else
	  return false;
}
**/

#define MAX_INT_DIGITS 8 // Maximum number of digits in int32 (and float)
extern "C" {
float __floatunsisf (unsigned long);
}


// Extracts a floating point value from a string. The following code is based loosely on
// the avr-libc strtod() function by Michael Stumpf and Dmitry Xmelkov and many freely
// available conversion method examples, but has been highly optimized for Grbl. For known
// CNC applications, the typical decimal value is expected to be in the range of E0 to E-4.
// Scientific notation is officially not supported by g-code, and the 'E' character may
// be a g-code word on some CNC systems. So, 'E' notation will not be recognized. 
// NOTE: Thanks to Radu-Eosif Mihailescu for identifying the issues with using strtod().
int read_float(char *line, uint8_t *char_counter, float *float_ptr)                  
{
  char *ptr = line + *char_counter;
  unsigned char c;
    
  // Grab first character and increment pointer. No spaces assumed in line.
  c = *ptr++;
  
  // Capture initial positive/minus character
  bool isnegative = false;
  if (c == '-') {
    isnegative = true;
    c = *ptr++;
  } else if (c == '+') {
    c = *ptr++;
  }
  
  // Extract number into fast integer. Track decimal in terms of exponent value.
  uint32_t intval = 0;
  int8_t exp = 0;
  uint8_t ndigit = 0;
  bool isdecimal = false;
  while(1) {
    c -= '0';
    if (c <= 9) {
      ndigit++;
      if (ndigit <= MAX_INT_DIGITS) {
        if (isdecimal) { exp--; }
        intval = (((intval << 2) + intval) << 1) + c; // intval*10 + c
      } else {
        if (!(isdecimal)) { exp++; }  // Drop overflow digits
      }
    } else if (c == (('.'-'0') & 0xff)  &&  !(isdecimal)) {
      isdecimal = true;
    } else {
      break;
    }
    c = *ptr++;
  }
  
  // Return if no digits have been read.
  if (!ndigit) { return(false); };
  
  // Convert integer into floating point.
  float fval;
  fval = __floatunsisf(intval);
  
  // Apply decimal. Should perform no more than two floating point multiplications for the
  // expected range of E0 to E-4.
  if (fval != 0) {
    while (exp <= -2) {
      fval *= 0.01; 
      exp += 2;
    }
    if (exp < 0) { 
      fval *= 0.1; 
    } else if (exp > 0) {
      do {
        fval *= 10.0;
      } while (--exp > 0);
    } 
  }

  // Assign floating point value with correct sign.    
  if (isnegative) {
    *float_ptr = -fval;
  } else {
    *float_ptr = fval;
  }

  *char_counter = ptr - line - 1; // Set char_counter to next statement
  
  return(true);
}
