#ifndef HPGL_PARSER_H
#define HPGL_PARSER_H
/*
  hpgl_parser.h - hpgl module
  Part of DiyouPCBFirm

  Copyright (c) 2013 V.Ayllon.  All rights reserved.

  Inspired on HP2xx:

  Parts Copyright (c) 1991 - 1994 Heinz W. Werntges.  All rights reserved.
  Parts Copyright (c) 1998-2002,2003 Martin Kroeker  All rights reserved.

DiyouPCBFirm is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
to anyone for the consequences of using it or for whether it serves any
particular purpose or works at all, unless he says so in writing.  Refer
to the GNU General Public License, Version 2 or later, for full details.

Everyone is granted permission to copy, modify and redistribute
DiyouPCBFirm, but only under the conditions described in the GNU General Public
License.  A copy of this license is supposed to have been
given to you along with DiyouPCB so you can know your rights and
responsibilities.  It should be in a file named COPYING.  Among other
things, the copyright notice and this notice must be preserved on all
copies.

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/
#define P1X_default	  0.0	/* drop margins       */
#define P1Y_default	  0.0
#define P2X_default   100.	
#define P2Y_default   100.

#define PLOT_UNIT	0.025f

#define MAX_PARAMETERS 4

// PA command parameters

#define X_COORD	0
#define Y_COORD 1

// AA command parameters

#define I_COORD 0
#define J_COORD 1
#define ARC_THETA 2
#define SEG_ALPHA 3

// IP command parameters

#define IP_P1X	0
#define IP_P1Y	1
#define IP_P2X	2
#define IP_P2Y	3

// SC command parameters

#define SC_P1X	0
#define SC_P1Y	1
#define SC_P2X	2
#define SC_P2Y	3


/* Supported HPGL commands, ASCII-coded as High-byte/low-byte int's */
#define LCMD	0x404C	// Not a HPGL command used for laser on/off
#define PCMD	0x4050	// Not a HPGL command used for laser power
#define HYCMD	0x4048	// Not a HPGL command used for BL setting
#define STCMD	0x4049	// Not a HPGL command used for Status reporting
#define HP_AA	0x4141
#define HP_IN	0x494E
#define HP_IP	0x4950
#define HP_PA	0x5041
#define HP_PD	0x5044
#define HP_PU	0x5055
#define HP_PW	0x5057
#define HP_QL	0x514C
#define HP_SC	0x5343
#define HP_SP	0x5350
#define HP_VS	0x5653
#define HP_LO	0x4c4F  // Not an HPGL command

// Command error
#define PARSER_ERROR 0xffff

extern float numpar[MAX_PARAMETERS];

void hpgl_init();
uint16_t hpgl_parser_run(char* buffer);
uint16_t hpgl_parse_command(uint16_t command);
bool get_IP_coordinates();
bool get_SC_coordinates();
bool get_single_value();
bool get_PA_coordinates();
bool get_AA_coordinates();
bool get_HY_values();
int read_float(char *line, uint8_t *char_counter, float *float_ptr);
float hpgl_user_to_plotter(float coord, uint8_t axis);
float hpgl_plotter_to_user(float coord, uint8_t axis);


#endif
